@@ -676,16 +676,16 @@ magnifier_get_display_rect_bounds (Magnifier *magnifier, GNOME_Magnifier_RectBou
 static void magnifier_adjust_source_size (Magnifier *magnifier)
 {
 	GNOME_Magnifier_RectBounds rect_bounds;	
+	gdouble vfract_top, vfract_bottom, hfract_left, hfract_right;
 	magnifier_get_display_rect_bounds (magnifier, &rect_bounds, FALSE);
-	gdouble vfract, hfract;
-	hfract = (double) (magnifier->target_bounds.x2 - magnifier->target_bounds.x1) / (double) (rect_bounds.x2 - rect_bounds.x1);
-	vfract = (double) (magnifier->target_bounds.y2 - magnifier->target_bounds.y1) / (double) (rect_bounds.y2 - rect_bounds.y1);
-	if (vfract > hfract) /* vertical splitpane, approximately */
+	hfract_left = (double) (magnifier->target_bounds.x1) / (double) rect_bounds.x2;
+	vfract_top = (double) (magnifier->target_bounds.y1) / (double) rect_bounds.y2;
+	hfract_right = (double) (rect_bounds.x2 - magnifier->target_bounds.x2) / (double) rect_bounds.x2;
+	vfract_bottom = (double) (rect_bounds.y2 - magnifier->target_bounds.y2) / (double) rect_bounds.y2;
+	/* we make our 'source' rectangle the largest available subsection which we aren't occupying */
+	if (MAX (hfract_left, hfract_right) > MAX (vfract_top, vfract_bottom))  /* vertical split, approximately */
 	{
-	        gint leftw, rightw;
-		leftw = magnifier->target_bounds.x1 - rect_bounds.x1;
-		rightw = rect_bounds.x2 - magnifier->target_bounds.x2;
-		if (rightw > leftw) {
+		if (hfract_right > hfract_left) {
 			magnifier->source_bounds.x1 = magnifier->target_bounds.x2;
 			magnifier->source_bounds.x2 = rect_bounds.x2;
 		}
@@ -697,12 +697,9 @@ static void magnifier_adjust_source_size (Magnifier *magnifier)
 		magnifier->source_bounds.y1 = rect_bounds.y1;
 		magnifier->source_bounds.y2 = rect_bounds.y2;
 	}
-	else
+	else /* more-or-less horizontally split */
 	{
-	        gint toph, bottomh;
-		toph = magnifier->target_bounds.y1 - rect_bounds.y1;
-		bottomh = rect_bounds.y2 - magnifier->target_bounds.y2;
-		if (bottomh > toph) {
+		if (vfract_bottom > vfract_top) {
 			magnifier->source_bounds.y1 = magnifier->target_bounds.y2;
 			magnifier->source_bounds.y2 = rect_bounds.y2;
 		}
@@ -1912,6 +1909,14 @@ magnifier_get_cursor (Magnifier *magnifier)
 		}
 		g_free (default_cursor_filename);
 	    }
+#ifdef HAVE_XFIXES
+	    else {
+		GdkPixbuf *cursor_pixbuf = magnifier_get_source_pixbuf (magnifier);
+		magnifier_set_cursor_from_pixbuf (magnifier, cursor_pixbuf);
+		if (cursor_pixbuf) g_object_unref (cursor_pixbuf);
+		magnifier_transform_cursor (magnifier);
+	    }
+#endif
         }
 	return magnifier->priv->cursor;
 }
