@@ -1173,8 +1173,8 @@ zoom_region_paint_pixmap (ZoomRegion *zoom_region,
 		gdk_draw_drawable (zoom_region->priv->w->window,
 				   zoom_region->priv->default_gc,
 				   zoom_region->priv->pixmap,
-				   area->x + zoom_region->priv->exposed_bounds.x1,
-				   area->y + zoom_region->priv->exposed_bounds.y1,
+				   area->x + zoom_region->priv->exposed_bounds.x1 - zoom_region->priv->source_area.x1 * zoom_region->xscale,
+				   area->y + zoom_region->priv->exposed_bounds.y1 - zoom_region->priv->source_area.y1 * zoom_region->yscale,
 				   area->x,
 				   area->y,
 				   area->width,
@@ -1727,7 +1727,6 @@ zoom_region_get_source_subwindow (ZoomRegion *zoom_region,
 		}
 		else
 		{
-			Magnifier *magnifier = (Magnifier *) zoom_region->priv->parent;
 			zoom_region->priv->source_drawable = gdk_screen_get_root_window (
 				gdk_display_get_screen (
 					magnifier->source_display,
@@ -1842,8 +1841,8 @@ zoom_region_update (ZoomRegion *zoom_region,
 				     zoom_region->priv->scaled_pixbuf,
 				     0,
 				     0,
-				     paint_rect.x + zoom_region->priv->exposed_bounds.x1,
-				     paint_rect.y + zoom_region->priv->exposed_bounds.y1,
+				     paint_rect.x + zoom_region->priv->exposed_bounds.x1 - zoom_region->priv->source_area.x1 * zoom_region->xscale,
+				     paint_rect.y + zoom_region->priv->exposed_bounds.y1 - zoom_region->priv->source_area.y1 * zoom_region->yscale,
 				     paint_rect.width,
 				     paint_rect.height,
 				     GDK_RGB_DITHER_NONE,
@@ -2327,6 +2326,15 @@ zoom_region_set_viewport (ZoomRegion *zoom_region,
 		if (zoom_region->priv->pixmap) 
 			g_object_unref (zoom_region->priv->pixmap);
 		zoom_region_create_pixmap (zoom_region);
+		if (zoom_region->priv->scaled_pixbuf)
+		        g_object_unref (zoom_region->priv->scaled_pixbuf);
+
+		zoom_region->priv->scaled_pixbuf = 
+		  gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8,
+				  (zoom_region->priv->source_area.x2 -
+				   zoom_region->priv->source_area.x1) * zoom_region->xscale + 1,
+				  (zoom_region->priv->source_area.y2 -
+				   zoom_region->priv->source_area.y1) * zoom_region->yscale + 1);
 	}
 	zoom_region_queue_update (zoom_region,
 				  zoom_region_source_rect_from_view_bounds (
